@@ -10,8 +10,8 @@ function (W, S, v, log = FALSE) {
         log.gammapart <- log.gammapart + lgamma(0.5 * (v + 1 - i))
     }
     log.denom <- log.gammapart + (0.5 * v * k) * log(2) + (0.25 * k * (k - 1)) * log(pi)
-    log.detS <- determinant(S, logarithm = TRUE)$modulus
-    log.detW <- determinant(W, logarithm = TRUE)$modulus
+    log.detS <- c(determinant(S, logarithm = TRUE)$modulus)
+    log.detW <- c(determinant(W, logarithm = TRUE)$modulus)
     trace <- sum(diag(solve(S, W)))
     log.num <- - 0.5 * (v * log.detS - (v - k - 1) * log.detW + trace)
     if (log) 
